﻿<#
.SYNOPSIS

  AD2Hostpool.ps1 - version 01 - 20191030/Stefan Peters
  Synchronizes WSAD Group members, to WVD HostPools

.DESCRIPTION

  AD2Hostpool.ps1 script will synchronize the UPN names of the WSAD group members to the WVD Hostpools.

.PREPARE
  
 - Install the required powershell modules "Install-Module -Name Microsoft.RDInfra.RDPowerShell" and "Add-WindowsFeature RSAT-AD-PowerShell"
 - Put an encrypted password in the .secpwd file.
 - Change line 26,27 and 28
 - Command to create .secpwd file: Read-host -prompt "Enter password to be encrypted in a file " -assecurestring | convertfrom-securestring | out-file "c:\scripts\Password.secpwd"

#>

# Import required modules
Import-Module Microsoft.RDInfra.RDPowerShell 
Import-Module ActiveDirectory
# / Import required modules

# Parameters
$tenantname="CONTOSO"
$azureAccountName = "admin@tenantname.onmicrosoft.com"
$azurePassword    = cat "c:\scripts\Password.secpwd" | ConvertTo-SecureString  
#
$global:ScriptName           = $MyInvocation.MyCommand.Name                              # current script name
$global:ScriptNameAndPath    = $MyInvocation.MyCommand.Definition                        # put full file and pathname in var.
$global:ScriptDate           = (Get-Date).ToString("yyyyMM")
$global:ScriptPath           = $global:ScriptNameAndPath.Replace($global:ScriptName, "") # current script path
$global:ScriptINI            = $global:ScriptNameAndPath.Replace($global:ScriptName, "")+"AD2Hostpool.csv" # current script path
$global:ScriptLogPath        = $global:ScriptPath+""      # log path
$global:Logfilename          = $global:Scriptlogpath+$global:ScriptDate+"_"+$global:ScriptName+".log"
#
$csv=import-csv -Path $global:ScriptINI -Delimiter ";" 
#
Function LOG($strText) 
 {
   $log_csv=";"
   $log_StrDatum=(Get-Date).ToString("yyyyMMdd HH:mm:ss") 
   $log_logline=""      
   Write-Host -ForegroundColor green $strText   
   $log_logline=$log_StrDatum+$log_csv+$strText
    Add-content -Path $global:logfilename $log_logline
} #/log

#Init 
if (!(test-path $Global:Logfilename)) { $logfile = New-Item -ItemType file -Path $Global:Logfilename } else { $logfile = $global:Logfilename }  

# Logon to RDSAccount 
$psCred = New-Object System.Management.Automation.PSCredential($azureAccountName, $azurePassword)
Add-RdsAccount -DeploymentUrl "https://rdbroker.wvd.microsoft.com" -Credential $pscred | out-null
# /Logon to RDSAccount 

Foreach ($csvitem in $csv) # Parse through the CSV.
  {    
    $LocalADGroup=$($csvitem.wsadgroup)
    $hostpoolname=$($csvitem.hostpoolname)
    Write-host -ForegroundColor green "Synchronizing AD Group: $localadgroup with WVDPool : $hostpoolname"    
    $hpexist=$null
    $xhpexist=Get-RdsHostPool -TenantName $tenantname -name $hostpoolname -ErrorAction SilentlyContinue
    $hpexist=[bool]$xhpexist
    If ($hpexist)
      {
        $syncaantal=0
        LOG "Hostpool check: $($hostpoolname) does exist, syncing accounts"
        $LocalADMembers=Get-ADGroupMember -Identity $LocalADGroup        
        $appgroup=get-rdsappgroup -TenantName $tenantname -HostPoolName $hostpoolname
        $appgroupname=$appgroup.AppGroupName
        $appgroupusers=$null
        $appgroupusers=(Get-RdsAppGroupUser -TenantName $tenantname -HostPoolName $hostpoolname -AppGroupName $appgroupname).UserPrincipalName        
        $xADUPN=@()        
        Foreach ($aduser in $LocalADMembers)
          {    
            $xTempUPN=(Get-ADUser -Identity $aduser).userprincipalname
            $xADUPN+=$xTempUPN
            LOG "  AD UPN: $($xTempUPN)"
          }                
        $add=$null
        $remove=$null        
        $add=$xADUPN | where {$appgroupusers -notcontains $_}
        $remove=$appgroupusers | Where {$xADUPN -notcontains $_}       
        If ($add.count -gt 0)
        {        
          Foreach ($xAdd in $add)
            {
              Write-Host "Add-RdsAppGroupUser -TenantName $tenantname -HostPoolName $hostpoolname -AppGroupName $appgroupname -UserPrincipalName $xadd"
              LOG "  Add-RdsAppGroupUser: $($xAdd)"
              Add-RdsAppGroupUser -TenantName $tenantname -HostPoolName $hostpoolname -AppGroupName $appgroupname -UserPrincipalName $xadd 
              $syncaantal++
            }
        }        
        If ($remove.count -gt 0)
        {         
          Foreach ($xRemove in $remove)
            {
              Write-host "Remove-RdsAppGroupUser -TenantName $tenantname -HostPoolName $hostpoolname -AppGroupName $appgroupname -UserPrincipalName $xremove"
              LOG "  Remove-RdsAppGroupUser: $($xremove)"
              Remove-RdsAppGroupUser -TenantName $tenantname -HostPoolName $hostpoolname -AppGroupName $appgroupname -UserPrincipalName $xremove    
              $syncaantal++
            }          
        }
    If ($syncaantal -ne 0)
      { 
        # Something was synced
        LOG "  Number of changes: $($syncaantal)"
      } else
      {
         # Nothing was synced
        LOG "  Nothing changed: $($syncaantal)"
      }
   } else 
   {
      LOG "Hostpool check: $($hostpoolname) does not exist, skip."
      Write-host -ForegroundColor Yellow "$hostpoolname does not exist"
   }

} #/Foreach - Parse through the CSV